
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

/*  =============================================
	xvis user interface builder for Athena widgets using
	xv class library.  

	Object Segment

	V1.0BETA  - last edited 25/06/96
    ============================================= */

// ---------------------------------------------
// Include file declarations

#include "xvis.h"
#include <string.h>
#include <stdio.h>

// --------------------------------------------
// Function Declarations

void func_popup_delobj(Widget aw, XtPointer client, XtPointer call);
void func_popup_resize(Widget aw, XtPointer client, XtPointer call);
void func_popup_redraw(Widget aw, XtPointer client, XtPointer call);
void func_popup_calls(Widget aw, XtPointer client, XtPointer call);
void func_popup_events(Widget aw, XtPointer client, XtPointer call);
void func_popup_copy(Widget aw, XtPointer client, XtPointer call);

void copyResources(int source, int destination);
void DeleteNamedObject(String name);
void DeleteScan(int parentObject);
void DeleteObject(int thisobject);
void buildObjectPopup(String name, objectType otype);
int ObjectByName(String name);
void SetCounterValue();

void placeSizer();
void removeSizer();

void size_call(Widget aw, XtPointer client, XtPointer call);
void move_call(Widget aw, XtPointer client, XtPointer call);

void motion_call(Widget aw, XtPointer client, register XMotionEvent *event, Boolean *cont);
void stop_call(Widget aw, XtPointer client, register XButtonEvent *event, Boolean *cont);

void enter_call(Widget aw, XtPointer client, register XEnterWindowEvent *event, Boolean *cont);
void leave_call(Widget aw, XtPointer client, register XLeaveWindowEvent *event, Boolean *cont);
void MotionStop();

extern void SetLoadedResources(int obj);

// --------------------------------------------
// Local data

char *popupmenu[]= {	"Redraw this Object" , "Delete this object", "Resize/Move this object",
				 	"Calls...", "Events...", "Copy Resources..."};
XtCallbackProc popupprocs[]= {	func_popup_redraw, func_popup_delobj, func_popup_resize,
							func_popup_calls, func_popup_events, func_popup_copy };
Boolean in_motion=FALSE;

Boolean is_sizing=FALSE;
 
// --------------------------------------------
// Callbacks

// ************
// handle Calls... selection
//
void func_popup_calls(Widget aw, XtPointer client, XtPointer call)
{
	buildCallsWindow(CALLMODE);
}

void func_popup_events(Widget aw, XtPointer client, XtPointer call)
{
	buildCallsWindow(EVENTMODE);
}

void func_copy_thisone(Widget aw, XtPointer client, XtPointer call)
{
	xvisObjList.RemoveGrab();
	xvisObjList.Close();
	copyResources(selected_ref, CurrentObject);
}

void func_popup_copy(Widget aw, XtPointer client, XtPointer call)
{
	buildObjectListWindow(func_copy_thisone);
	openObjectListWindow(CurrentObjectType, CurrentObject);
}

// ************
// delobj handles the object delete option from the popup menu
//
void func_delobj_yes(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
	DeleteNamedObject(CurrentObjectName);
	SetupPropertiesList(CurrentParent, TRUE);
	if (nobject==0) {
		controlEventsData(FALSE);
		xvisObjectMenu.EntryResource("Remap Object...", XtNsensitive, FALSE);
		xvisObjectMenu.EntryResource("New/Edit Menu", XtNsensitive, FALSE);
		xvisEditMenu.SetSensitive(FALSE);
		xvisFileMenu.EntryResource("Save Form As...", XtNsensitive, FALSE);
		xvisFileMenu.EntryResource("Save Project", XtNsensitive, FALSE);
		xvisFileMenu.EntryResource("Save Form", XtNsensitive, FALSE);
		xvisGenerateMenu.EntryResource("Build", XtNsensitive, FALSE);
		xvisGenerateMenu.EntryResource("ReBuild All", XtNsensitive, FALSE);
		xvisSave.SetSensitive(FALSE);
	}
}

void func_delobj_no(Widget aw, XtPointer client, XtPointer call)
{
	alert.Close();
}

void func_popup_delobj(Widget aw, XtPointer client, XtPointer call)
{
	if (CurrentObjectType==XVWindow) {
		alert.init("Delete Form...", xvisControl, 
			"You have chosen to delete a form\n Are you sure you want to do this?\n(Deleting the form will also delete ALL of it's children!)",
			XVMSGSIMPLECHOICE, "green",
			func_delobj_yes, func_delobj_no);
		alert.Open();

	}
	else {
		DeleteNamedObject(CurrentObjectName);
		SetupPropertiesList(CurrentParent, TRUE);
		if (nobject==0) {
			controlEventsData(FALSE);
			xvisObjectMenu.EntryResource("Remap Object...", XtNsensitive, FALSE);
			xvisObjectMenu.EntryResource("New/Edit Menu", XtNsensitive, FALSE);
			xvisEditMenu.SetSensitive(FALSE);
			xvisFileMenu.EntryResource("Save Form As...", XtNsensitive, FALSE);
			xvisFileMenu.EntryResource("Save Project", XtNsensitive, FALSE);
			xvisFileMenu.EntryResource("Save Form", XtNsensitive, FALSE);
			xvisGenerateMenu.EntryResource("Build", XtNsensitive, FALSE);
			xvisGenerateMenu.EntryResource("ReBuild All", XtNsensitive, FALSE);
			xvisSave.SetSensitive(FALSE);
		}
	}
}

// *************
// resize handles a re-size request from the popup menu
//
void func_popup_resize(Widget aw, XtPointer client, XtPointer call)
{

	if (ObjectBeingSized != -1) {
		removeSizer();
		ObjectBeingSized=-1;
	}
	placeSizer();
}

// ********************
// redraw handles the popup menu redraw option
//
void func_popup_redraw(Widget aw, XtPointer client, XtPointer call)
{
	XtUnmapWidget(objects[CurrentObject].pcore->ID());
	XtMapWidget(objects[CurrentObject].pcore->ID());
}

// ********************
// size_call handles the object Sizer resize callback
//
void size_call(Widget aw, XtPointer client, XtPointer call)
{
	sizer->SizerHide(*CurrentObjectWindow);

	objects[ObjectBeingSized].pcore->Unmap();

	sizer->SizerShow(*CurrentObjectWindow);				
	sizer->SizerSize(*CurrentObjectWindow, client);
	is_sizing=TRUE;
}

// ********************
// move_call handles the object Sizer move callback
//
void move_call(Widget aw, XtPointer client, XtPointer call)
{
	sizer->SizerHide(*CurrentObjectWindow);

	objects[ObjectBeingSized].pcore->Unmap();

	sizer->SizerShow(*CurrentObjectWindow);				
	sizer->SizerMove(*CurrentObjectWindow, client);
	is_sizing=FALSE;
}

// --------------------------------------------
// Events

// **********************
// expose is the Expose service routine  (xvWindow class only)
//
void winexpose(Widget aw, XtPointer client, register XExposeEvent *event, Boolean *cont)
{
	char *s=(char *) client;
	int obj=ObjectByName(s);
	drawGrid(objects[obj].parent_window);
}

// **********************
// enter_call is the EnterNotify service routine  (xvWindow class only)
//
void enter_call(Widget aw, XtPointer client, register XEnterWindowEvent *event, Boolean *cont)
{
}

// *********************
// leave_call is the LeaveNotify service routine  (xvWindow class only)
//
void leave_call(Widget aw, XtPointer client, register XLeaveWindowEvent *event, Boolean *cont)
{
	if ((event->detail != 2) && (ObjectBeingSized != -1)) { 
		if (in_motion) MotionStop();
		removeSizer();
		ObjectBeingSized=-1;
	}
}

// **********************
// WindowEvent handles the StructureNotify event (xvWindow class only)
//
void WindowEvent(Widget aw, XtPointer client, register XConfigureEvent *event, Boolean *cont)
{
	if (event->type==ConfigureNotify) {
		strcpy(CurrentObjectName, (char *) client);
		CurrentObject=ObjectByName(CurrentObjectName);
		CurrentObjectType=objects[CurrentObject].object_type;	
		CurrentToolType=ToolNameToType(CurrentToolName);
		CurrentClass=objects[CurrentObject].pcore;
		CurrentParent=objects[CurrentObject].parent_index;
		CurrentParentType=objects[CurrentObject].parent_type;
		CurrentObjectWindow=objects[objects[CurrentObject].parent_window].pwindow;
	
		SetupPropertiesList(CurrentObject, TRUE);
		
		if (objects[CurrentObject].hasmenu) {
			objects[CurrentObject].menubar->Unmap(); 
			objects[CurrentObject].menubar->SetWidth(
				objects[CurrentObject].pwindow->GetWidth()-7);
			objects[CurrentObject].menubar->Map();
		}
		drawGrid(objects[CurrentObject].parent_window);
	}
}

// **************************
// ObjectButtonEvent services all ButtonEvents for all objects
//
void ObjectButtonEvent(Widget aw, XtPointer client, register XButtonEvent *event, Boolean *cont)
{
	unsigned int b;
	Window rw, cw;

	char *temptool=(char *) xPointer.GetCurrent();
	if (temptool==NULL) {
		xPointer.ChangeCurrent(objectName[XVPointer]);	
		CurrentToolName=(char *) xPointer.GetCurrent();
	}	
	strcpy(CurrentObjectName, (char *) client);
	CurrentObject=ObjectByName(CurrentObjectName);
	CurrentObjectType=objects[CurrentObject].object_type;	
	CurrentToolType=ToolNameToType(CurrentToolName);
	CurrentClass=objects[CurrentObject].pcore;
	CurrentParent=objects[CurrentObject].parent_index;
	CurrentParentType=objects[CurrentObject].parent_type;
	CurrentObjectWindow=objects[objects[CurrentObject].parent_window].pwindow;
	
	SetupPropertiesList(CurrentObject, TRUE);

	XQueryPointer(XtDisplay(aw), XtWindow(objects[CurrentObject].object_widget), &rw, &cw,
		&CurrentX, &CurrentY, &RelativeX, &RelativeY, &b);

	if ((CurrentParentType==XVBox) || (CurrentParentType==XVForm)) {
		RelativeX=CurrentX;
		RelativeY=CurrentY;
	}

	if (snapToGrid) {
		RelativeX=(RelativeX/gridX) * gridX;
		RelativeY=(RelativeY/gridY) * gridY;
		CurrentX=(CurrentX/gridX) * gridX;
		CurrentY=(CurrentY/gridY) * gridY;
	}

	if (event->button== Button3) {
		if (ObjectBeingSized != -1) {
			removeSizer();
			ObjectBeingSized=-1;
		}
		if (objectpopupopen) {
			objectpopup.Close();
			objectpopupopen=FALSE;
		}

		buildObjectPopup(CurrentObjectName, CurrentObjectType);
		objectpopup.Open(CurrentX, CurrentY);
		objectpopupopen=TRUE;
	}
	if (event->button== Button1) {

		if (objectpopupopen) {
			objectpopup.Close();
			objectpopupopen=FALSE;
		}
		
		switch(CurrentObjectType) {
			case XVWindow:
			case XVBox:
			case XVForm:
				if (ObjectBeingSized!=-1) {
					removeSizer();
					ObjectBeingSized=-1;
				}
				else
					if (CurrentToolType != XVPointer) {
						CreateObject(CurrentToolType);
						CurrentToolName=objectName[XVPointer];
						xPointer.ChangeCurrent(CurrentToolName);
						toollabel.SetLabel(CurrentToolName);
						toollabel.SetWidth(xvisTools.GetWidth());
						toollabel.SetBackground("SkyBlue");
						placeSizer();
					}
				break;
			case XVList:
			case XVButton:
			case XVRepeater:
			case XVBitmap:
			case XVLabel:
			case XVDropDown:
			case XVEditField:
			case XVToggle:
			case XVVScroll:
			case XVHScroll:
			case XVFileList:
			case XVDirList:
			case XVPanner:
				if (ObjectBeingSized != -1) { 
					removeSizer();
					if (ObjectBeingSized != CurrentObject) 
						placeSizer();
 					else
						ObjectBeingSized=-1;
				}
				else 	{
					placeSizer();
				}		
			default:
				break;
		}
	}		
	drawGrid(objects[CurrentObject].parent_window);
}

// ************************
// Remove the sizer from the screen
//
void removeSizer()
{
	sizer->SizerHide(*CurrentObjectWindow);
	sizer->SizerUnhook(*CurrentObjectWindow);
	delete sizer;
}

// ************************
// Compute sizer XY position depending on object type (called by placeSizer)
//
void ComputeActualXY(int obj, int *sx, int *sy)
{
	*sx=0;
	*sy=0;
	while (objects[obj].parent_type !=XVNone) {

		if (objects[obj].parent_type == XVBox) {
			*sx=*sx+(int)objects[obj].pcore->GetX();
			*sy=*sy+(int)objects[obj].pcore->GetY();
		}
		else {
			*sx=*sx+(int)objects[obj].pcore->GetHorizDistance();
			*sy=*sy+(int)objects[obj].pcore->GetVertDistance();
		}

	obj=objects[obj].parent_index;
	}
}		

// ******************************
// placeSizer on the screen
//	
void placeSizer()
{

	int sx, sy;
	int mode=XVSIZEMOVE;
	int is_sizing=FALSE;
	ObjectBeingSized=CurrentObject;
	ObjectBeingSizedParent=CurrentParent;
	ObjectBeingSizedParentType=CurrentParentType;

	if ((CurrentParentType == XVBox) || (CurrentParentType == XVForm)) 
		ComputeActualXY(ObjectBeingSized, &sx, &sy);
	else
		objects[ObjectBeingSized].pcore->GetDistance(&sx, &sy);
	
	sizer = new xvSizer(
		*CurrentObjectWindow, 
		sx, sy, 
		objects[ObjectBeingSized].pcore->GetWidth() +
			2*objects[ObjectBeingSized].pcore->GetBorderWidth(),
		objects[ObjectBeingSized].pcore->GetHeight() +
			2*objects[ObjectBeingSized].pcore->GetBorderWidth(),
		mode, 8,
		size_call, move_call, 
		(XtEventHandler) motion_call, (XtEventHandler) stop_call);
				
	sizer->SizerShow(*CurrentObjectWindow);				

}

// ***************************
// Motion event handler for sizer grips
//
void motion_call(Widget aw, XtPointer client, register XMotionEvent *event, Boolean *cont)
{
	in_motion=TRUE;
	sizer->SizerMotion(*CurrentObjectWindow);
}

// ***************************
// Stop event handler for sizer grips
//
void stop_call(Widget aw, XtPointer client, register XButtonEvent *event, Boolean *cont)
{
	MotionStop();
	is_sizing=FALSE;
}

// **************************
// Returns object to screen with new size/position.. called by stop_call and leave_call
//
void MotionStop()
{
	int x, y, w, h, px, py;

	sizer->SizerStop(*CurrentObjectWindow);
	drawGrid(objects[ObjectBeingSized].parent_window);

	sizer->SizerInfo(&x, &y, &w, &h);
	if (snapToGrid) {
		if (! is_sizing) {
			x=(x/gridX) * gridX;
			y=(y/gridY) * gridY;
		}
		else {
			w=(w/gridX) * gridX;
			h=(h/gridY) * gridY;
			if (w<=0) w=gridX;
			if (h<=0) h=gridY;
		}
	}
	if ((ObjectBeingSizedParentType==XVBox) || (ObjectBeingSizedParentType==XVForm)) {
		ComputeActualXY(ObjectBeingSizedParent, &px, &py);
		x=x-px;
		y=y-py;
		if (ObjectBeingSizedParentType==XVBox) 
			objects[ObjectBeingSized].pcore->SetXY(x, y);
		else	{
			objects[ObjectBeingSized].pcore->SetHorizDistance(x);
			objects[ObjectBeingSized].pcore->SetVertDistance(y);	
		}
	}
	else {
		objects[ObjectBeingSized].pcore->SetHorizDistance(x);
		objects[ObjectBeingSized].pcore->SetVertDistance(y);	
		objects[ObjectBeingSized].pcore->SetXY(x, y);
	}
	objects[ObjectBeingSized].pcore->SetSize(w, h);
	if (objects[ObjectBeingSized].object_type == XVDropDown)
		objects[ObjectBeingSized].plabel->SetHeight(24);
	sizer->SizerHide(*CurrentObjectWindow);
	objects[ObjectBeingSized].pcore->Map();

	int wid=objects[ObjectBeingSized].pcore->GetWidth();
	int hgt=objects[ObjectBeingSized].pcore->GetHeight();
	int sx=0, sy=0;
	if ((CurrentParentType == XVBox) || (CurrentParentType == XVForm)) 
		ComputeActualXY(ObjectBeingSized, &sx, &sy);
	else
		objects[ObjectBeingSized].pcore->GetDistance(&sx, &sy);

	sizer->SizerWarp(*CurrentObjectWindow, sx, sy, wid, hgt);

	sizer->SizerShow(*CurrentObjectWindow);				
	SetupPropertiesList(ObjectBeingSized, FALSE);
	in_motion=FALSE;
}

// --------------------------------------------
// Build Object Popup Menu for requested objectType

void buildObjectPopup(String name, objectType otype)
{
	objectpopup.init("objectPopup", CurrentObjectName, 6, popupmenu, 
		popupprocs, xvisContext);

	if (CurrentObjectType==XVWindow)
		objectpopup.EntryResource("Resize/Move this object", XtNsensitive, FALSE);

	if (object_call_number[CurrentObjectType]==0)
		objectpopup.EntryResource("Calls...", XtNsensitive, FALSE);		

	if (objects[CurrentObject].events==0L)
		objectpopup.EntryResource("Events...", XtNsensitive, FALSE);		

	if (objectInstance[CurrentObjectType] < 2)
		objectpopup.EntryResource("Copy Resources...", XtNsensitive, FALSE);		
		
}

// --------------------------------------------
// Create an object of type 'objectType'

int CreateObject(objectType object)
{
	char buf[40];
	int thisobject, pe, vcounter=1, objx=0;

	if (nobject<MAXOBJECTS) {
		thisobject=firstAvailableSlot();
		while (objx != -1) {
			sprintf(buf, "%s%d", objectName[object], objectInstance[object]+vcounter);
			objx=ObjectByName(buf);
			if (objx != -1) vcounter++;
		}

		objects[thisobject].object_type=object;
		strcpy(objects[thisobject].name, buf);
		if (CurrentObject==-1) {
			objects[thisobject].parent_type=XVNone;
			objects[thisobject].parent_index=-1;
			objects[thisobject].parent_window=thisobject;
			objects[thisobject].num_children=0;
		}
		else {
			objects[thisobject].parent_type=CurrentObjectType;
			objects[thisobject].parent_index=CurrentObject;
			objects[thisobject].parent_window=objects[CurrentObject].parent_window;
			objects[CurrentObject].num_children++;
		}		
		objects[thisobject].object_index=objectInstance[object];
		objects[thisobject].events=0L;
		objects[thisobject].object_ref1=-1;
		objects[thisobject].object_ref2=-1;
		objects[thisobject].object_ref3=-1;
		objects[thisobject].totalmenu=0;
		objects[thisobject].nummenu=0;
		objects[thisobject].hasmenu=FALSE;
		objects[thisobject].fixed_size=FALSE;
		objects[thisobject].iconic=FALSE;
		objects[thisobject].open_at_startup=FALSE;
		objects[thisobject].has_bitmap=FALSE;
		objects[thisobject].has_leftbitmap=FALSE;
		objects[thisobject].enabled=TRUE;
		objects[thisobject].visible=TRUE;
		
		objects[thisobject].edit_mode=0;
		objects[thisobject].resize_mode=0;
		objects[thisobject].vscroll_mode=0;
		objects[thisobject].hscroll_mode=0;
		objects[thisobject].wrap_mode=0;
		objects[thisobject].echo=TRUE;
		objects[thisobject].autowrap=FALSE;
		objects[thisobject].ctrl=FALSE;
		objects[thisobject].shape=0;
		objects[thisobject].justify=0;

		if (objectInstance[object]==0)
			objects[thisobject].open_at_startup=TRUE;

		strcpy(objects[thisobject].background_colour, "(default)");
		strcpy(objects[thisobject].foreground_colour, "(default)");
		strcpy(objects[thisobject].border_colour, "(default)");
		strcpy(objects[thisobject].topshadow_colour, "(default)");
		strcpy(objects[thisobject].bottomshadow_colour, "(default)");
		strcpy(objects[thisobject].menu_foreground, "(default)");
		strcpy(objects[thisobject].menu_background, "(default)");
		strcpy(objects[thisobject].obj_cursor, "(None)");
		strcpy(objects[thisobject].leftbitmap, "(None)");

		strcpy(objects[thisobject].obj_path, "");
		strcpy(objects[thisobject].obj_icon, "");
		strcpy(objects[thisobject].obj_font, "");
		strcpy(objects[thisobject].obj_bitref, "(None)");
		strcpy(objects[thisobject].obj_leftbitref, "(None)");

		for (int i=0; i<3; i++)
			objects[thisobject].callbackcode[i]=NULL;

		for (int i=0; i<16; i++)
			objects[thisobject].eventcode[i]=NULL;
		
		switch(object) {
			case XVTimer:
				objects[thisobject].ptimer = new xvTimer(objects[thisobject].name, 
					*CurrentClass, 0L, func_timer);			
				objects[thisobject].ptimer->SetDistance(0, 0);
				objects[thisobject].pcore->SetXY(0, 0);
				objects[thisobject].object_widget=objects[thisobject].ptimer->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(FALSE);
				break; 
			case XVList:
				objects[thisobject].plist = new xvVisualList(objects[thisobject].name, 
					*CurrentClass, func_dummy, 1, listvalues, 1);			
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].plist->xvList::SetSize(100,100);
				objects[thisobject].plist->xvView::ManageChildren();
				objects[thisobject].plist->xvView::SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].plist->xvView::ID();

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");
		
				break;
			case XVButton:
				strcpy(objects[thisobject].obj_path, "(None)");
				objects[thisobject].pbutton = new xvButton(objects[thisobject].name, *CurrentClass, func_dummy);			
				objects[thisobject].pbutton->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].pbutton->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].object_bitmap=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);
				objects[thisobject].leftbit=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVRepeater:
				strcpy(objects[thisobject].obj_path, "(None)");
				objects[thisobject].prepeater = new xvRepeater(objects[thisobject].name, *CurrentClass, func_dummy, NULL, func_dummy, func_dummy);			
				objects[thisobject].prepeater->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].prepeater->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].object_bitmap=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);
				objects[thisobject].leftbit=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVBitmap:
				strcpy(objects[thisobject].obj_path, "(None)");
				objects[thisobject].pbitmap = new xvBitmap(objects[thisobject].name, 
					*CurrentClass);			
				objects[thisobject].pbitmap->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].pbitmap->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				break;
			case XVLabel:
				strcpy(objects[thisobject].obj_path, "(None)");
				objects[thisobject].plabel = new xvLabel(objects[thisobject].name, *CurrentClass);			
				objects[thisobject].plabel->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].plabel->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].object_bitmap=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);
				objects[thisobject].leftbit=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVDropDown:
				objects[thisobject].plabel = new xvLabel(objects[thisobject].name, *CurrentClass);
				objects[thisobject].plabel->SetHeight(24);
				objects[thisobject].plabel->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].plabel->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");
				objects[thisobject].plabel->SetHeight(24);

				break;
			case XVEditField:
				objects[thisobject].peditfield = new xvEditField(objects[thisobject].name, 
					*CurrentClass, XawtextRead, 25, 100, XawAsciiString, objects[thisobject].name);			
				objects[thisobject].peditfield->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].peditfield->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVBox:
				objects[thisobject].pbox = new xvBox(objects[thisobject].name, 
					*CurrentClass);			
				objects[thisobject].pbox->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].pbox->SetSize(100,100);
				objects[thisobject].object_widget=objects[thisobject].pbox->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				break;
			case XVToggle:
				strcpy(objects[thisobject].obj_path, "(None)");
				objects[thisobject].ptoggle = new xvToggle(objects[thisobject].name, 
					*CurrentClass, func_dummy, objects[thisobject].name);			
				objects[thisobject].ptoggle->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].ptoggle->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].object_bitmap=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);
				objects[thisobject].leftbit=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pcore, FALSE);

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVForm:
				objects[thisobject].pform = new xvVisualForm(objects[thisobject].name, 
					*CurrentClass);			
				objects[thisobject].pform->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].pform->ID();
				objects[thisobject].pform->SetSize(100,100);
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				break;
			case XVVScroll:
				objects[thisobject].pvscroll = new xvScrollbar(objects[thisobject].name, 
					*CurrentClass, XtorientVertical, 200, 
					func_dummy, func_dummy, func_dummy);			
				objects[thisobject].pvscroll->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].pvscroll->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				break;
			case XVHScroll:
				objects[thisobject].phscroll = new xvScrollbar(objects[thisobject].name, 
					*CurrentClass, XtorientHorizontal, 200, 
					func_dummy, func_dummy, func_dummy);			
				objects[thisobject].phscroll->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].phscroll->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				break;
			case XVPanner:
				objects[thisobject].ppanner = new xvPanner(objects[thisobject].name, 
					*CurrentClass, func_dummy, 100, 100, 50, 50);			
				objects[thisobject].ppanner->SetDistance(RelativeX, RelativeY);
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].ppanner->SetSize(100,100);
				objects[thisobject].object_widget=objects[thisobject].ppanner->ID();
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				break;
			case XVFileList:
				strcpy(objects[thisobject].obj_path, cwd);
				objects[thisobject].pfilelist= new xvFileList(objects[thisobject].name, 
					*CurrentClass, objects[thisobject].obj_path, XVFILELIST, 
					FALSE, func_dummy, FALSE);
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].pfilelist->xvList::SetSize(150,180);
				objects[thisobject].pfilelist->xvView::ManageChildren();
				objects[thisobject].pfilelist->xvView::SetDistance(RelativeX, RelativeY);				
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].pfilelist->xvView::ID();

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVDirList:
				strcpy(objects[thisobject].obj_path, cwd);
				objects[thisobject].pdirlist= new xvFileList(objects[thisobject].name, 
					*CurrentClass, objects[thisobject].obj_path, XVDIRLIST, 
					FALSE, func_dummy, FALSE);
				objects[thisobject].pcore->SetFixedPosition(TRUE);
				objects[thisobject].pcore->SetResizable(TRUE);
				objects[thisobject].pdirlist->xvList::SetSize(150,180);
				objects[thisobject].pdirlist->xvView::ManageChildren();
				objects[thisobject].pdirlist->xvView::SetDistance(RelativeX, RelativeY);				
				objects[thisobject].pcore->SetXY(RelativeX, RelativeY);
				objects[thisobject].object_widget=objects[thisobject].pdirlist->xvView::ID();

				objects[thisobject].object_font= new xvFont(*objects[thisobject].pcore, 
					"*fixed-*-r-*-100-*-70*");
				objects[thisobject].pcore->SetResource(XtNfont, 
					(XtArgVal) objects[thisobject].object_font->FontRef());
				strcpy(objects[thisobject].obj_font, "*fixed-*-r-*-100-*-70*");

				break;
			case XVWindow:
				strcpy(objects[thisobject].obj_icon, "(None)");
				objects[thisobject].pwindow = new xvWindow(objects[thisobject].name,
						objects[thisobject].name , xvisContext);
				objects[thisobject].pwindow->SetSize(300, 300);				
				objects[thisobject].winwidth=300;
				objects[thisobject].winheight=300;
				objects[thisobject].pwindow->SetWindowResource(
					XtNgeometry, (XtArgVal)"+310+260");
				objects[thisobject].winxpos=310;
				objects[thisobject].winypos=260;
				objects[thisobject].pwindow->Open();
				objects[thisobject].object_bitmap=new xvBitmap(
					objects[thisobject].name, *objects[thisobject].pwindow, FALSE);
				objects[thisobject].object_widget=objects[thisobject].pwindow->ID();

				objects[thisobject].pwindow->GCSetForeground(
					stringToPixel(*objects[thisobject].pwindow, "navy"));
				objects[thisobject].pwindow->GCSetBackground(
					stringToPixel(*objects[thisobject].pwindow, "navy"));

				objects[thisobject].pwindow->AddEvent(StructureNotifyMask,
					(XtEventHandler) WindowEvent,
					(XtPointer) objects[thisobject].name);

				objects[thisobject].pwindow->AddEvent(EnterWindowMask,
					(XtEventHandler) enter_call,
					(XtPointer) objects[thisobject].name);

				objects[thisobject].pwindow->AddEvent(LeaveWindowMask,
					(XtEventHandler) leave_call,
					(XtPointer) objects[thisobject].name);

				objects[thisobject].pwindow->AddEvent(ExposureMask,
					(XtEventHandler) winexpose,
					(XtPointer) objects[thisobject].name);

				pe=findFirstProjectEntry();
				project_list[pe]=thisobject;
				updateProjectListWindow();
				drawGrid(thisobject);
				break;
			default:
				break;
		}
		strcpy(objects[thisobject].obj_menu_font, "*times-*-r-*-140*");
	
		objects[thisobject].pcore->AddEvent(ButtonPressMask, 
				(XtEventHandler) ObjectButtonEvent, 
				(XtPointer) objects[thisobject].name);

		if (uses3D[object])
			objects[thisobject].pcore->SetBeNiceToColormap(FALSE);

		objectInstance[object]++;
		SetupPropertiesList(nobject, TRUE);
		nobject++;
		if (nobject==1) {
			controlEventsData(TRUE);
			xvisObjectMenu.EntryResource("Remap Object...", XtNsensitive, TRUE);
			xvisObjectMenu.EntryResource("New/Edit Menu", XtNsensitive, TRUE);
			xvisEditMenu.SetSensitive(TRUE);
			xvisFileMenu.EntryResource("Save Form As...", XtNsensitive, TRUE);
			if (strlen(project_name)>0) xvisFileMenu.EntryResource("Save Project", XtNsensitive, TRUE);
			xvisFileMenu.EntryResource("Save Form", XtNsensitive, TRUE);
			xvisSave.SetSensitive(TRUE);
		}
		SetCounterValue();
		CurrentObject=thisobject;
	}
}

// -------------------------------------------
// Delete named Object 

void DeleteNamedObject(String name)
{
	int o=ObjectByName(name);
	if (o!=-1) {
		DeleteScan(o);
		DeleteObject(o);
	}
}

// -------------------------------------------
// Scan structure for objects with parent value, delete 

void DeleteScan(int parentObject)
{
	objectType ob;

	for (int i=0; i<MAXOBJECTS; i++) {
		ob=objects[i].object_type;
		if ((i != parentObject) && (ob != XVNone))
			if (objects[i].parent_index == parentObject) { 
				if ((ob == XVWindow) || (ob == XVBox) || (ob==XVForm))
					DeleteScan(i); 
				DeleteObject(i);
			}
	}
}

// -------------------------------------------
// Delete Object referenced by 'thisobject'

void DeleteObject(int thisobject)
{
	if (nobject<=0) return;
	if (thisobject>=MAXOBJECTS) return;
	
	objects[thisobject].pwindow->RemoveEvent(ButtonPressMask, 
			(XtEventHandler) ObjectButtonEvent, 
			(XtPointer) objects[thisobject].name);

	objectType object=objects[thisobject].object_type;
	objects[thisobject].object_type=XVNone;

	if (CurrentParent != -1) objects[CurrentParent].num_children--;

	if (object == XVNone) return;
	nobject--;
	
	switch(object) {
		case XVTimer:
			objects[thisobject].ptimer->Hide();
			delete objects[thisobject].ptimer;
			break;
		case XVList:
			delete objects[thisobject].object_font;
			objects[thisobject].plist->xvView::Hide();
			delete objects[thisobject].plist;
			break;
		case XVButton:
			delete objects[thisobject].object_font;
			delete objects[thisobject].object_bitmap;
			delete objects[thisobject].leftbit;
			objects[thisobject].pbutton->Hide();
			delete objects[thisobject].pbutton;
			break;
		case XVRepeater:
			delete objects[thisobject].object_font;
			delete objects[thisobject].object_bitmap;
			delete objects[thisobject].leftbit;
			objects[thisobject].prepeater->Hide();
			delete objects[thisobject].prepeater;
			break;
		case XVBitmap:
			objects[thisobject].pbitmap->Hide();
			delete objects[thisobject].pbitmap;
			break;
		case XVLabel:
			delete objects[thisobject].object_font;
			delete objects[thisobject].object_bitmap;
			delete objects[thisobject].leftbit;
			objects[thisobject].plabel->Hide();
			delete objects[thisobject].plabel;
			break;
		case XVDropDown:
			delete objects[thisobject].object_font;
			objects[thisobject].plabel->Hide();
			delete objects[thisobject].plabel;
			break;
		case XVEditField:
			delete objects[thisobject].object_font;
			objects[thisobject].peditfield->Hide();
			delete objects[thisobject].peditfield;
			break;
		case XVBox:
			objects[thisobject].pbox->Hide();
			delete objects[thisobject].pbox;
			break;
		case XVToggle:
			delete objects[thisobject].object_font;
			delete objects[thisobject].object_bitmap;
			delete objects[thisobject].leftbit;
			objects[thisobject].ptoggle->Hide();
			delete objects[thisobject].ptoggle;
			break;
		case XVForm:
			objects[thisobject].pform->Hide();
			delete objects[thisobject].pform;
			break;
		case XVVScroll:
			objects[thisobject].pvscroll->Hide();
			delete objects[thisobject].pvscroll;
			break;
		case XVHScroll:
			objects[thisobject].phscroll->Hide();
			delete objects[thisobject].phscroll;
			break;
		case XVPanner:
			objects[thisobject].ppanner->Hide();
			delete objects[thisobject].ppanner;
			break;
		case XVFileList:
			delete objects[thisobject].object_font;
			objects[thisobject].pfilelist->xvView::Hide();
			delete objects[thisobject].pfilelist;
			break;
		case XVDirList:
			delete objects[thisobject].object_font;
			objects[thisobject].pdirlist->xvView::Hide();
			delete objects[thisobject].pdirlist;
			break;
		case XVWindow:
			if (objects[thisobject].hasmenu) {
				for (int i=0; i<objects[thisobject].totalmenu; i++)
					if (objects[thisobject].menuinfo[i].codebuffer != NULL) {
						delete objects[thisobject].menuinfo[i].codebuffer;
						objects[thisobject].menuinfo[i].codebuffer=NULL;
					}
				delete objects[thisobject].menuinfo;

				delete objects[thisobject].menubar;
				for (int i=0; i<objects[thisobject].nummenu; i++)
					delete objects[thisobject].pulldown[i];
			}

			project_list[findFirstProjectEntry(thisobject)]=-1;
			updateProjectListWindow();

			objects[thisobject].pwindow->RemoveEvent(EnterWindowMask,
				(XtEventHandler) enter_call,
				(XtPointer) objects[thisobject].name);

			objects[thisobject].pwindow->RemoveEvent(LeaveWindowMask,
				(XtEventHandler) leave_call,
				(XtPointer) objects[thisobject].name);

			objects[thisobject].pwindow->RemoveEvent(StructureNotifyMask,
				(XtEventHandler) WindowEvent,
				(XtPointer) objects[thisobject].name);

			objects[thisobject].pwindow->RemoveEvent(ExposureMask,
				(XtEventHandler) winexpose,
				(XtPointer) objects[thisobject].name);

			objects[thisobject].pwindow->Close();
			delete objects[thisobject].object_bitmap;
			delete objects[thisobject].pwindow;
			break;
		default:
			break;
	}

	for (int i=0; i<MAXEDITWINDOWS; i++) 
		if ((editor[i].in_use) && (editor[i].object==thisobject)) {
			delete editor[i].editor;
			delete editor[i].edit_buffer;
		}	

	for (int i=0; i<NUMEVENTS; i++)
		if (objects[thisobject].eventcode[i] != NULL) 
			delete objects[thisobject].eventcode[i];

	for (int i=0; i<3; i++)
		if (objects[thisobject].callbackcode[i] != NULL) 
			delete objects[thisobject].callbackcode[i];

	SetCounterValue();
}

// **************************
// Copy resources from one object to another.
// Objects are of the same type.
//
void copyResources(int source, int destination)
{
	strcpy(objects[destination].obj_cursor, objects[source].obj_cursor);
	strcpy(objects[destination].obj_font, objects[source].obj_font);
	strcpy(objects[destination].obj_path, objects[source].obj_path);
	strcpy(objects[destination].obj_icon, objects[source].obj_icon);
	strcpy(objects[destination].obj_bitref, objects[source].obj_bitref);
	strcpy(objects[destination].leftbitmap, objects[source].leftbitmap);
	strcpy(objects[destination].obj_leftbitref, objects[source].obj_leftbitref);

	strcpy(objects[destination].background_colour, objects[source].background_colour);
	strcpy(objects[destination].foreground_colour, objects[source].foreground_colour);
	strcpy(objects[destination].border_colour, objects[source].border_colour);
	strcpy(objects[destination].topshadow_colour, objects[source].topshadow_colour);
	strcpy(objects[destination].bottomshadow_colour, objects[source].bottomshadow_colour);

	for (int i=0; i<3; i++) {
		if (objects[destination].callbackcode[i]!=NULL) 
			delete objects[destination].callbackcode[i];

		objects[destination].callbackcode[i]=NULL;

		if (objects[source].callbackcode[i]!=NULL) {
			objects[destination].callbackcode[i]=new char[strlen(objects[source].callbackcode[i])];
			strcpy(objects[destination].callbackcode[i], objects[source].callbackcode[i]);
		}
	}

	for (int i=0; i<16; i++) {
		if (objects[destination].eventcode[i]!=NULL) 
			delete objects[destination].eventcode[i];

		objects[destination].eventcode[i]=NULL;

		if (objects[source].eventcode[i]!=NULL) {
			objects[destination].eventcode[i]=new char[strlen(objects[source].eventcode[i])];
			strcpy(objects[destination].eventcode[i], objects[source].eventcode[i]);
		}
	}

	objects[destination].edit_mode=objects[source].edit_mode;
	objects[destination].resize_mode=objects[source].resize_mode;
	objects[destination].vscroll_mode=objects[source].vscroll_mode;
	objects[destination].hscroll_mode=objects[source].hscroll_mode;
	objects[destination].wrap_mode=objects[source].wrap_mode;
	objects[destination].shape=objects[source].shape;
	objects[destination].justify=objects[source].justify;
	objects[destination].object_ref1=objects[source].object_ref1;
	objects[destination].object_ref2=objects[source].object_ref2;
	objects[destination].object_ref3=objects[source].object_ref3;
	objects[destination].object_time=objects[source].object_time;

	objects[destination].autowrap=objects[source].autowrap;
	objects[destination].echo=objects[source].echo;
	objects[destination].ctrl=objects[source].ctrl;
	objects[destination].enabled=objects[source].enabled;
	objects[destination].visible=objects[source].visible;
	objects[destination].changed=objects[source].changed;
	objects[destination].fixed_size=objects[source].fixed_size;
	objects[destination].hasmenu=objects[source].hasmenu;
	objects[destination].iconic=objects[source].iconic;
	objects[destination].open_at_startup=objects[source].open_at_startup;
	objects[destination].has_bitmap=objects[source].has_bitmap;
	objects[destination].has_leftbitmap=objects[source].has_leftbitmap;
	objects[destination].events=objects[source].events;
	objects[destination].winxpos=objects[source].winxpos;
	objects[destination].winypos=objects[source].winypos;
	objects[destination].winwidth=objects[source].winwidth;
	objects[destination].winheight=objects[source].winheight;

	if (objects[destination].hasmenu) {
		strcpy(objects[destination].obj_menu_font, objects[source].obj_menu_font);
		objects[destination].totalmenu=objects[source].totalmenu;
		objects[destination].nummenu=objects[source].nummenu;
		for (int i=0; i<MAXPULLDOWNS; i++)
			objects[destination].numpulldown[i]=objects[source].numpulldown[i];

		strcpy(objects[destination].menu_background, objects[source].menu_background);
		strcpy(objects[destination].menu_foreground, objects[source].menu_foreground);
	}

//	Now GET source's resources

	objectType otype=objects[source].object_type;
	int numres=object_resource_number[otype];
	char* *types=object_resource_types[otype];
	resourceType *datatypes=object_data_types[otype];

	for (int i=0; i<numres; i++) {
		resource_values[i].r_restype=types[i];
		resource_values[i].r_type=datatypes[i];
	}
	recoverResources(numres, source);		

//	Now SET dest. resources

	prop_object_index=destination;
	prop_object_type=otype;
	for (int i=0; i<numres-2; i++) {
		resource_values[i].r_restype=types[i];
		resource_values[i].r_type=datatypes[i];
		if ((strcmp(resource_values[i].r_restype, xvNobjectName)!=0) && (strcmp(resource_values[i].r_restype, xvNwindowName)!=0)) {
			strcpy(propeditbuffer, resource_values[i].r_value);
			selected_index=i;
			objects[destination].pcore->Unmap();
			setResourceExec(FALSE);		
			objects[destination].pcore->Map();
		}
	}
}

